@@ -1,10 +1,12 @@
-/* $Id: dbdimp.c,v 1.30 2003/01/27 21:50:53 matt Exp $ */
+/* $Id: dbdimp.c,v 1.33 2003/03/06 19:38:52 matt Exp $ */
 
 #include "SQLiteXS.h"
 
 DBISTATE_DECLARE;
 
-STRLEN myPL_na;
+#ifndef SvPV_nolen
+#define SvPV_nolen(x) SvPV(x,PL_na)
+#endif
 
 void
 sqlite_init(dbistate_t *dbistate)
@@ -13,20 +15,18 @@ sqlite_init(dbistate_t *dbistate)
     DBIS = dbistate;
 }
 
-void
-sqlite_error(SV *h, int rc, char *what)
+static void
+sqlite_error(SV *h, imp_xxh_t *imp_xxh, int rc, char *what)
 {
     dTHR;
-    D_imp_xxh(h);
 
     SV *errstr = DBIc_ERRSTR(imp_xxh);
     sv_setiv(DBIc_ERR(imp_xxh), (IV)rc);
     sv_setpv(errstr, what);
-    DBIh_EVENT2(h, ERROR_event, DBIc_ERR(imp_xxh), errstr);
     
-    if (dbis->debug >= 2) {
-        fprintf(DBILOGFP, "%s error %d recorded: %s\n",
-            what, rc, SvPV(errstr, myPL_na));
+    if (DBIS->debug >= 3) {
+        PerlIO_printf(DBILOGFP, "sqlite error %d recorded: %s\n",
+            rc, SvPV_nolen(errstr));
     }
 }
 
@@ -34,30 +34,36 @@ int
 sqlite_db_login(SV *dbh, imp_dbh_t *imp_dbh, char *dbname, char *user, char *pass)
 {
     dTHR;
+    int retval;
     char *errmsg = NULL;
 
+    if (DBIS->debug >= 3) {
+        PerlIO_printf(DBILOGFP, "    login '%s' (version %s, encoding %s)\n",
+            dbname, sqlite_version, sqlite_encoding);
+    }
+
     if ((imp_dbh->db = sqlite_open(dbname, 0, &errmsg)) == NULL) {
-        sv_setpv(DBIc_ERRSTR(imp_dbh), errmsg);
+	sqlite_error(dbh, (imp_xxh_t*)imp_dbh, 1, errmsg);
         Safefree(errmsg);
         return FALSE;
     }
+    DBIc_IMPSET_on(imp_dbh);
 
     imp_dbh->in_tran = FALSE;
     imp_dbh->no_utf8_flag = FALSE;
 
     sqlite_busy_timeout(imp_dbh->db, SQL_TIMEOUT);
 
-    if (sqlite_exec(imp_dbh->db, "PRAGMA empty_result_callbacks = ON",
+    if (retval = sqlite_exec(imp_dbh->db, "PRAGMA empty_result_callbacks = ON",
         NULL, NULL, &errmsg)
         != SQLITE_OK)
     {
         /*  warn("failed to set pragma: %s\n", errmsg); */
-        sv_setpv(DBIc_ERRSTR(imp_dbh), errmsg);
+	sqlite_error(dbh, (imp_xxh_t*)imp_dbh, retval, errmsg);
         Safefree(errmsg);
         return FALSE;
     }
 
-    DBIc_IMPSET_on(imp_dbh);
     DBIc_ACTIVE_on(imp_dbh);
 
     return TRUE;
@@ -73,7 +79,6 @@ sqlite_db_disconnect (SV *dbh, imp_dbh_t *imp_dbh)
         sqlite_db_rollback(dbh, imp_dbh);
     }
 
-
     sqlite_close(imp_dbh->db);
     imp_dbh->db = NULL;
 
@@ -97,19 +102,14 @@ sqlite_db_rollback(SV *dbh, imp_dbh_t *imp_dbh)
     int retval;
     char *errmsg;
 
-    if (DBIc_is(imp_dbh, DBIcf_AutoCommit)) {
-        warn("rollback ineffective with AutoCommit");
-        return TRUE;
-    }
-
     if (imp_dbh->in_tran) {
         if (retval = sqlite_exec(imp_dbh->db, "ROLLBACK TRANSACTION",
             NULL, NULL, &errmsg)
             != SQLITE_OK)
         {
-            sv_setpv(DBIc_ERRSTR(imp_dbh), errmsg);
+	    sqlite_error(dbh, (imp_xxh_t*)imp_dbh, retval, errmsg);
             Safefree(errmsg);
-            return retval;
+            return FALSE;
         }
         imp_dbh->in_tran = FALSE;
     }
@@ -134,9 +134,9 @@ sqlite_db_commit(SV *dbh, imp_dbh_t *imp_dbh)
             NULL, NULL, &errmsg)
             != SQLITE_OK)
         {
-            sv_setpv(DBIc_ERRSTR(imp_dbh), errmsg);
+	    sqlite_error(dbh, (imp_xxh_t*)imp_dbh, retval, errmsg);
             Safefree(errmsg);
-            return retval;
+            return FALSE;
         }
         imp_dbh->in_tran = FALSE;
     }
@@ -147,8 +147,7 @@ int
 sqlite_discon_all(SV *drh, imp_drh_t *imp_drh)
 {
     dTHR;
-    /* no way to do this. Just return OK */
-    return TRUE;
+    return FALSE; /* no way to do this */
 }
 
 void
@@ -165,7 +164,7 @@ sqlite_st_parse_sql(imp_sth_t *imp_sth, char *statement)
     /* warn("parsing: %s\n", statement); */
 
     while (*statement) {
-        /*  warn("parse: %c => %s\n", *statement, SvPV(chunk, myPL_na)); */
+        /*  warn("parse: %c => %s\n", *statement, SvPV_nolen(chunk)); */
         if (*statement == '\'') {
             if (in_literal) {
                 /*  either end of literal, or escape */
@@ -213,8 +212,6 @@ sqlite_st_prepare (SV *sth, imp_sth_t *imp_sth,
     dTHR;
     D_imp_dbh_from_sth;
 
-    sv_setpv(DBIc_ERRSTR(imp_dbh), "");
-
     imp_sth->nrow = 0;
     imp_sth->ncols = 0;
     imp_sth->params = newAV();
@@ -227,7 +224,7 @@ sqlite_st_prepare (SV *sth, imp_sth_t *imp_sth,
 char *
 sqlite_quote(imp_dbh_t *imp_dbh, SV *val)
 {
-    char *cval = SvPV(val, myPL_na);
+    char *cval = SvPV_nolen(val);
     SV *ret = sv_2mortal(NEWSV(0, SvCUR(val) + 2));
     if (strchr(cval, '\'')) {
         sv_setpvn(ret, "", 0);
@@ -241,7 +238,7 @@ sqlite_quote(imp_dbh_t *imp_dbh, SV *val)
             }
             *cval++;
         }
-        return SvPV(ret, myPL_na);
+        return SvPV_nolen(ret);
     }
     else {
         return cval;
@@ -262,14 +259,16 @@ sqlite_st_execute (SV *sth, imp_sth_t *imp_sth)
 
     /* warn("execute\n"); */
 
-    sv_setpv(DBIc_ERRSTR(imp_dbh), "");
+    if (DBIc_ACTIVE(imp_sth)) {
+        sqlite_st_finish(sth, imp_sth);
+    }
 
     sql = sv_2mortal(newSVsv(AvARRAY(imp_sth->sql)[pos++]));
 
     for (i = 0; i < num_params; i++) {
         SV *value = av_shift(imp_sth->params);
         if (value && SvOK(value)) {
-            /* warn("binding param: %s\n", SvPV(value, myPL_na)); */
+            /* warn("binding param: %s\n", SvPV_nolen(value); */
             sv_catpvn(sql, "'", 1);
             sv_catpv(sql, sqlite_quote(imp_dbh, value));
             sv_catpvn(sql, "'", 1);
@@ -283,40 +282,41 @@ sqlite_st_execute (SV *sth, imp_sth_t *imp_sth)
         }
         sv_catsv(sql, AvARRAY(imp_sth->sql)[pos++]);
     }
-    /* warn("Executing: %s;\n", SvPV(sql, myPL_na)); */
+    /* warn("Executing: %s;\n", SvPV_nolen(sql)); */
 
     if ( (!DBIc_is(imp_dbh, DBIcf_AutoCommit)) && (!imp_dbh->in_tran) ) {
         if (retval = sqlite_exec(imp_dbh->db, "BEGIN TRANSACTION",
             NULL, NULL, &errmsg)
             != SQLITE_OK)
         {
-            sqlite_error(sth, retval, errmsg);
-            /* sv_setpv(DBIc_ERRSTR(imp_dbh), errmsg); */
+            sqlite_error(sth, (imp_xxh_t*)imp_sth, retval, errmsg);
             Safefree(errmsg);
-            return -1;
+            return -2;
         }
         imp_dbh->in_tran = TRUE;
     }
 
     imp_sth->results = NULL;
-    if (retval = sqlite_get_table(imp_dbh->db, SvPV(sql, myPL_na), &(imp_sth->results),
+    if (retval = sqlite_get_table(imp_dbh->db, SvPV_nolen(sql), &(imp_sth->results),
         &(imp_sth->nrow), &(imp_sth->ncols), &errmsg) != SQLITE_OK)
     {
-        /* warn("exec failed: %s\n", errmsg); */
-        sqlite_error(sth, retval, errmsg);
+        sqlite_error(sth, (imp_xxh_t*)imp_sth, retval, errmsg);
         Safefree(errmsg);
         return -2;
     }
 
-    /* warn("exec ok - %d rows\n", imp_sth->nrow); */
     DBIc_NUM_FIELDS(imp_sth) = imp_sth->ncols;
     imp_sth->c_row = 1;
     if (imp_sth->ncols != 0) {
         DBIc_ACTIVE_on(imp_sth);
     }
     else {
+	/* sqlite_get_table doesn't set nrow for non-selects, probably a bug, */
+	/* so we have to call sqlite_changes explicitly here */
+	imp_sth->nrow = sqlite_changes(imp_dbh->db);
         sqlite_free_table(imp_sth->results);
     }
+    /* warn("exec ok - %d rows, %d cols\n", imp_sth->nrow, imp_sth->ncols); */
     DBIc_IMPSET_on(imp_sth);
     return imp_sth->nrow;
 }
@@ -335,7 +335,7 @@ sqlite_bind_ph (SV *sth, imp_sth_t *imp_sth,
     if (is_inout) {
         croak("InOut bind params not implemented");
     }
-    /* warn("bind: %s => %s\n", SvPV(param, myPL_na), SvPV(value, myPL_na)); */
+    /* warn("bind: %s => %s\n", SvPV_nolen(param), SvPV_nolen(value)); */
     if (sql_type >= SQL_NUMERIC && sql_type <= SQL_DOUBLE) {
         av_store(imp_sth->params, SvIV(param) - 1, newSVnv(SvNV(value)));
     }
@@ -375,12 +375,13 @@ sqlite_st_fetch (SV *sth, imp_sth_t *imp_sth)
                 val[len] = '\0';
             }
             sv_setpvn(AvARRAY(av)[i], val, len);
+
+	    if (!imp_dbh->no_utf8_flag) {
+		/* sv_utf8_encode(AvARRAY(av)[i]); */
+	    }
         }
         else {
-            sv_setsv(AvARRAY(av)[i], Nullsv);
-        }
-        if (!imp_dbh->no_utf8_flag && SvPOK(AvARRAY(av)[i])) {
-            /* sv_utf8_encode(AvARRAY(av)[i]); */
+            (void)SvOK_off(AvARRAY(av)[i]);
         }
     }
     imp_sth->c_row++;
@@ -412,17 +413,18 @@ int
 sqlite_st_blob_read (SV *sth, imp_sth_t *imp_sth,
                 int field, long offset, long len, SV *destrv, long destoffset)
 {
+    return 0;
 }
 
 int
 sqlite_db_STORE_attrib (SV *dbh, imp_dbh_t *imp_dbh, SV *keysv, SV *valuesv)
 {
     dTHR;
-    char *key = SvPV(keysv, myPL_na);
+    char *key = SvPV_nolen(keysv);
     char *errmsg;
     int retval;
 
-    if (strncmp(key, "AutoCommit", 10) == 0) {
+    if (strEQ(key, "AutoCommit")) {
         if (SvTRUE(valuesv)) {
             /* commit tran? */
             if ( (!DBIc_is(imp_dbh, DBIcf_AutoCommit)) && (imp_dbh->in_tran) ) {
@@ -430,9 +432,9 @@ sqlite_db_STORE_attrib (SV *dbh, imp_dbh_t *imp_dbh, SV *keysv, SV *valuesv)
                     NULL, NULL, &errmsg)
                     != SQLITE_OK)
                 {
-                    sv_setpv(DBIc_ERRSTR(imp_dbh), errmsg);
+		    sqlite_error(dbh, (imp_xxh_t*)imp_dbh, retval, errmsg);
                     Safefree(errmsg);
-                    return retval;
+                    return TRUE;
                 }
                 imp_dbh->in_tran = FALSE;
             }
@@ -440,7 +442,7 @@ sqlite_db_STORE_attrib (SV *dbh, imp_dbh_t *imp_dbh, SV *keysv, SV *valuesv)
         DBIc_set(imp_dbh, DBIcf_AutoCommit, SvTRUE(valuesv));
         return TRUE;
     }
-    else if (strncmp(key, "NoUTF8Flag", 10) == 0) {
+    else if (strEQ(key, "sqlite_no_utf8_flag") || strEQ(key, "NoUTF8Flag")) {
         warn("NoUTF8Flag is deprecated due to perl unicode weirdness\n");
         if (SvTRUE(valuesv)) {
             imp_dbh->no_utf8_flag = TRUE;
@@ -457,48 +459,31 @@ SV *
 sqlite_db_FETCH_attrib (SV *dbh, imp_dbh_t *imp_dbh, SV *keysv)
 {
     dTHR;
-    char *key = SvPV(keysv, myPL_na);
+    char *key = SvPV_nolen(keysv);
 
-    if (strncmp(key, "AutoCommit", 10) == 0) {
-        /* warn("fetching autocommit\n"); */
-        return newSViv(DBIc_is(imp_dbh, DBIcf_AutoCommit));
-    }
-    if (strncmp(key, "NoUTF8Flag", 10) == 0) {
+    if (strEQ(key, "sqlite_no_utf8_flag") || strEQ(key, "NoUTF8Flag")) {
         return newSViv(imp_dbh->no_utf8_flag ? 1 : 0);
     }
-    return NULL;
-}
-
-int
-sqlite_db_STORE_attrib_k (SV *dbh, imp_dbh_t *imp_dbh, SV *keysv, int dbikey, SV *valuesv)
-{
-    dTHR;
-    return FALSE;
-}
-
-SV *
-sqlite_db_FETCH_attrib_k (SV *dbh, imp_dbh_t *imp_dbh, SV *keysv, int dbikey)
-{
-    dTHR;
+    if (strEQ(key, "sqlite_version")) {
+        return newSVpv(sqlite_version, strlen(sqlite_version));
+    }
+    if (strEQ(key, "sqlite_encoding")) {
+        return newSVpv(sqlite_encoding, strlen(sqlite_encoding));
+    }
     return NULL;
 }
 
 int
 sqlite_st_STORE_attrib (SV *sth, imp_sth_t *imp_sth, SV *keysv, SV *valuesv)
 {
-    char *key = SvPV(keysv, myPL_na);
-
-    if (strEQ(key, "ChopBlanks")) {
-        DBIc_set(imp_sth, DBIcf_ChopBlanks, SvIV(valuesv));
-        return TRUE;
-    }
+    char *key = SvPV_nolen(keysv);
     return FALSE;
 }
 
 SV *
 sqlite_st_FETCH_attrib (SV *sth, imp_sth_t *imp_sth, SV *keysv)
 {
-    char *key = SvPV(keysv, myPL_na);
+    char *key = SvPV_nolen(keysv);
     SV *retsv = NULL;
     int i;
 
@@ -512,26 +497,18 @@ sqlite_st_FETCH_attrib (SV *sth, imp_sth_t *imp_sth, SV *keysv)
         AV *av = newAV();
         retsv = sv_2mortal(newRV(sv_2mortal((SV*)av)));
         while (--i >= 0) {
-            av_store(av, i, newSVpv(imp_sth->results[i], 0));
+            char *fieldname = imp_sth->results[i];
+            char *dot = instr(fieldname, ".");
+            if (dot) /* drop table name from field name */
+                fieldname = ++dot;
+            av_store(av, i, newSVpv(fieldname, 0));
         }
     }
     else if (strEQ(key, "NUM_OF_FIELDS")) {
         retsv = sv_2mortal(newSViv(imp_sth->ncols));
     }
-    else if (strEQ(key, "ChopBlanks")) {
-        retsv = sv_2mortal(newSViv(DBIc_has(imp_sth, DBIcf_ChopBlanks)));
-    }
 
     return retsv;
 }
 
-int
-sqlite_st_STORE_attrib_k (SV *sth, imp_sth_t *imp_sth, SV *keysv, int dbikey, SV *valuesv)
-{
-}
-
-SV *
-sqlite_st_FETCH_attrib_k (SV *sth, imp_sth_t *imp_sth, SV *keysv, int dbikey)
-{
-}
-
+/* end */
