@@ -12,7 +12,7 @@
 ** This file contains C code routines that are called by the parser
 ** to handle SELECT statements in SQLite.
 **
-** $Id: select.c,v 1.15 2003/01/27 21:50:54 matt Exp $
+** $Id: select.c,v 1.16 2003/03/04 07:51:44 matt Exp $
 */
 #include "sqliteInt.h"
 
@@ -697,7 +697,9 @@ static void generateColumnTypes(
 ){
   Vdbe *v = pParse->pVdbe;
   int i;
-  if( (pParse->db->flags & SQLITE_ReportTypes)==0 ) return;
+  if( pParse->useCallback && (pParse->db->flags & SQLITE_ReportTypes)==0 ){
+    return;
+  }
   for(i=0; i<pEList->nExpr; i++){
     Expr *p = pEList->a[i].pExpr;
     char *zType = 0;
@@ -1456,7 +1458,9 @@ static int multiSelect(Parse *pParse, Select *p, int eDest, int iParm){
 
   /* Issue a null callback if that is what the user wants.
   */
-  if( (pParse->db->flags & SQLITE_NullCallback)!=0 && eDest==SRT_Callback ){
+  if( eDest==SRT_Callback &&
+    (pParse->useCallback==0 || (pParse->db->flags & SQLITE_NullCallback)!=0)
+  ){
     sqliteVdbeAddOp(v, OP_NullCallback, p->pEList->nExpr, 0);
   }
   return 0;
@@ -2030,9 +2034,19 @@ int sqliteSelect(
   }
   if( pOrderBy ){
     for(i=0; i<pOrderBy->nExpr; i++){
+      int iCol;
       Expr *pE = pOrderBy->a[i].pExpr;
+      if( sqliteExprIsInteger(pE, &iCol) && iCol>0 && iCol<=pEList->nExpr ){
+        sqliteExprDelete(pE);
+        pE = pOrderBy->a[i].pExpr = sqliteExprDup(pEList->a[iCol-1].pExpr);
+      }
+      if( sqliteExprResolveIds(pParse, base, pTabList, pEList, pE) ){
+        goto select_end;
+      }
+      if( sqliteExprCheck(pParse, pE, isAgg, 0) ){
+        goto select_end;
+      }
       if( sqliteExprIsConstant(pE) ){
-        int iCol;
         if( sqliteExprIsInteger(pE, &iCol)==0 ){
           sqliteSetString(&pParse->zErrMsg, 
                "ORDER BY terms must not be non-integer constants", 0);
@@ -2046,25 +2060,16 @@ int sqliteSelect(
           pParse->nErr++;
           goto select_end;
         }
-        sqliteExprDelete(pE);
-        pE = pOrderBy->a[i].pExpr = sqliteExprDup(pEList->a[iCol-1].pExpr);
-      }
-      if( sqliteExprResolveIds(pParse, base, pTabList, pEList, pE) ){
-        goto select_end;
-      }
-      if( sqliteExprCheck(pParse, pE, isAgg, 0) ){
-        goto select_end;
       }
     }
   }
   if( pGroupBy ){
     for(i=0; i<pGroupBy->nExpr; i++){
+      int iCol;
       Expr *pE = pGroupBy->a[i].pExpr;
-      if( sqliteExprIsConstant(pE) ){
-        sqliteSetString(&pParse->zErrMsg, 
-             "GROUP BY expressions should not be constant", 0);
-        pParse->nErr++;
-        goto select_end;
+      if( sqliteExprIsInteger(pE, &iCol) && iCol>0 && iCol<=pEList->nExpr ){
+        sqliteExprDelete(pE);
+        pE = pGroupBy->a[i].pExpr = sqliteExprDup(pEList->a[iCol-1].pExpr);
       }
       if( sqliteExprResolveIds(pParse, base, pTabList, pEList, pE) ){
         goto select_end;
@@ -2072,6 +2077,21 @@ int sqliteSelect(
       if( sqliteExprCheck(pParse, pE, isAgg, 0) ){
         goto select_end;
       }
+      if( sqliteExprIsConstant(pE) ){
+        if( sqliteExprIsInteger(pE, &iCol)==0 ){
+          sqliteSetString(&pParse->zErrMsg, 
+               "GROUP BY terms must not be non-integer constants", 0);
+          pParse->nErr++;
+          goto select_end;
+        }else if( iCol<=0 || iCol>pEList->nExpr ){
+          char zBuf[2000];
+          sprintf(zBuf,"GROUP BY column number %d out of range - should be "
+             "between 1 and %d", iCol, pEList->nExpr);
+          sqliteSetString(&pParse->zErrMsg, zBuf, 0);
+          pParse->nErr++;
+          goto select_end;
+        }
+      }
     }
   }
 
@@ -2157,8 +2177,9 @@ int sqliteSelect(
   /* Do an analysis of aggregate expressions.
   */
   sqliteAggregateInfoReset(pParse);
-  if( isAgg ){
+  if( isAgg || pGroupBy ){
     assert( pParse->nAgg==0 );
+    isAgg = 1;
     for(i=0; i<pEList->nExpr; i++){
       if( sqliteExprAnalyzeAggregates(pParse, pEList->a[i].pExpr) ){
         goto select_end;
@@ -2306,7 +2327,9 @@ int sqliteSelect(
 
   /* Issue a null callback if that is what the user wants.
   */
-  if( (pParse->db->flags & SQLITE_NullCallback)!=0 && eDest==SRT_Callback ){
+  if( eDest==SRT_Callback &&
+    (pParse->useCallback==0 || (pParse->db->flags & SQLITE_NullCallback)!=0)
+  ){
     sqliteVdbeAddOp(v, OP_NullCallback, pEList->nExpr, 0);
   }
 
